#!/usr/bin/perl
# sendsms.pl
# 
# Send an SMS from the command line. The SMS is tunneled
# thru ICQ.
#
# The syntax is:
#
# $ ./sendsms.pl <mobile-no> <text>
#
# written by K. Nikhil Shankar <nikhilwiz@yahoo.com>
# parts borrowed from Robin Fisher <robin@phase3solutions.com> ICQ2000.pm
#
# Copyright (c) 2001, K. Nikhil Shankar
# Licensed under the GNU General Public License 2.0 (see COPYING).

use ICQ2000;
use Getopt::Std;

use vars qw/$opt_d/;

getopts('d');

my $mobileno=shift @ARGV || usage();
($mobileno) = ($mobileno =~ /(\d+)/);
my $text=shift @ARGV || usage();

if ($#ARGV != -1) {
	usage();
}

$SIG{INT}=\&disconnect;

my %config;

read_config_file();

my $icq = Net::ICQ2000->new($config{uin}, $config{password}, "1"); #the 1 tells the module to auto connect

if ($opt_d) {
	$icq->{_Debug} = 1;
}

push (@{$icq->{_Auto_Login_Contact_List}}, $config{uin});

$icq->Add_Hook("Srv_Mes_Received", \&RecMessage);
$icq->Add_Hook("Srv_Srv_Message", \&SrvMessage);

$icq->Add_Hook("Srv_GSC_User_Info", \&DisplayDetails);
#$icq->Add_Hook("Srv_GSC_MOTD", \&MOTD);
#$icq->Add_Hook("Srv_BLM_Contact_Online", \&User_Online);
#$icq->Add_Hook("Srv_BLM_Contact_Offline", \&DisplayDetails);


my $sms_sent=0;
my $connected_for_real=0;
my $deliverable_yes = 0;
my $sent_sms_id;

my $timeout = time() + 60;

while (time() < $timeout) {
    $icq->Execute_Once();
#    $icq->{_Connected}; (huh?)

    if ($connected_for_real) {
	if (!$sms_sent) {
		send_sms($mobileno, $text);
	}
	if ($deliverable_yes) {
		exit(0);
	}
    }

}

# We only got here because we timed out. Show this

print "Timed out..\n" if ($opt_d);
exit(4);

# -------------------------------------------------------------------------

sub disconnect {
    print "Exiting due to interrupt..\n" if ($opt_d);
    exit(3);
}

sub send_sms {

        my $mobileno = shift;
	my $details = { };

	$details->{SMS_Dest_Number} = "$mobileno"; #(country code, then mobile number..)
	$details->{text} = shift;
	$details->{MessageType} = "SMS";

	$icq->Send_Command("Cmd_Srv_Message", $details);
	print "sending to $mobileno:\n$details->{text}\n" if ($opt_d);
	$sms_sent=1;

}

sub RecMessage {
    my($Object, $details) = @_;

    if ($details->{MessageType} eq "sms_delivery_receipt"){
    #this message tells us they recieved our SMS message..

	if ($opt_d) {
	        print "SMS receipt:\n";

	        print "Delivered at $details->{delivery_time}\n"; #- Date we sent the message (GMT) (I know these two seem the wrong way around, but that's how the server gives them..)
	        print "Submitted at $details->{submition_time}\n"; #- Date they got the message (GMT)
	        print "Message ID: $details->{message_id}\n"; #- the SMS message's ID

	        print "Destination: $details->{destination}\n"; #- the number the orig message was sent to
	        print "Delivered: $details->{delivered}\n"; #- was it delivered? (Yes/No)
	        print "Excerpt: $details->{text}\n"; #- a bit of the message sent (first 20 chars)
	}

        if(($details->{message_id} eq $sent_sms_id) || (($details->{message_id} eq "") && ($details->{destination} eq $mobileno)) || ($details->{destination} eq "")) {

                if($details->{delivered} eq "Yes") {
                        print "SMS sent!\n" if ($opt_d);
			exit(0);
                } else {
			print "SMS failed!\n" if ($opt_d);
			exit(1);
		}
        }

    }
}

sub SrvMessage {
    my($Object, $details) = @_;
        #These are responces from the server which r unique to ICQ..
    if ($details->{MessageType} eq "sms_response"){
            #this is the return message confirming that the server got your SMS message, and can send it on..
	if ($opt_d) {
            print "SMS response:\n";
            print "Deliverable: $details->{deliverable}\n"; #     - is it deliverable? (Yes/No)
            print "Sending thru $details->{network}\n"; #- What company is sending the message
            print "Message ID: $details->{message_id}\n"; #- The ID given to the SMS message
    #        print $details->{messages_left}    , "\n"; #- Messages left to send (I always get 0..)
	}

            ($details->{deliverable} eq "Yes") || croak("Error: $details->{TaggedDataString}\n");
            #- The non decoded data string, log it if the message isn't deliverable, since it contains the error message (which isn't collected very verll otherwise..)

            $sent_sms_id=$details->{message_id};

	    $deliverable_yes = 1;
    }
}

sub DisplayDetails {
    my($Object, $details) = @_;

	#very generic handler that just print's out all the bit's of data that get passed to it..
	if ($opt_d) {

		foreach (keys %$details){
			print "[$_][$details->{$_}]\n";
		}
		print "\n";
	}

	
	$connected_for_real=1;

}

sub read_config_file {
	
	my ($line, $name, $val);
	%config = ( uin => 0, password => "", shell => "/bin/sh", logfile => "smsterm.log" );

	my $fn = "$ENV{HOME}/.smsterm.cfg";
	open FH, $fn or die "Error opening $fn: $!";
	
	for $line (<FH>) {
		$line =~ /(\w+)(\s*)=(\s*)(.*)/;
		$name = $1;
		$val = $4;
		$config{$name} = $val;
	}

	close FH;
}

sub usage {
	print STDERR "Usage:\n\n";
	print STDERR "sendsms.pl [-d] <mobileno> <text>\n";
	exit(1);
}
